# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package FDN::Web::Annuaire;

use Exporter;

@ISA = qw ( Exporter );

@EXPORT = qw (
    &initdata
    &getpage
    &setrewritetable
    &settemplate
    &delentry
    &addentry
    &modifyentry
    &getentry
    &searchentry
    &countentries
);

use strict;
use FDN::Config;
use FDN::Common::TemplateParse;

my $tablefile = undef;
my $tablename = undef;
my $tpl = undef;

####################################################################
####################################################################
## sub settemplate                                                ##
##                                                                ##
## Parametres :                                                   ##
##   $tpl = le nom du fichier contenant le template               ##
##                                                                ##
## Retour :                                                       ##
##   cette fonction ne retourne rien                              ##
####################################################################
####################################################################

sub settemplate($) {
	($tpl) = @_;
}

####################################################################
####################################################################
## sub setrewritetable                                            ##
##                                                                ##
## Parametres :                                                   ##
##  $tablefile = le fichier contenant la rewritetable             ##
##  $tablename = le nom de la table dans le fichier               ##
##                                                                ##
## Retour :                                                       ##
##   cette fonction ne retourne rien                              ##
####################################################################
####################################################################

sub setrewritetable($$) {
	($tablefile, $tablename) = @_;
}

####################################################################
####################################################################
## sub initdata                                                   ##
##                                                                ##
## Parametres :                                                   ##
##  $rawdata = les donnes brutes  afficher \@                   ##
##  $sortby = le nom de la key sur laquelle faire le tri          ##
##  $flfield = le nom de la key sur laquelle extraire la lettre   ##
##                                                                ##
## Retour :                                                       ##
##  un HREF contenant les infos  passer  la fonction            ##
##  d'affichage                                                   ##
####################################################################
####################################################################

sub initdata($$$)
{
	my ($rawdata, $sortby, $flfield) = @_;
	if ( (!defined $rawdata) or (ref $rawdata ne "ARRAY") ) {
		return;
	}
	my @tmp = sort { lc($a->{$sortby}) cmp lc($b->{$sortby}) } @$rawdata;
	my $data = \@tmp;
	my $byletter;
	my $letters;
	for (my $i = 0; $i < scalar @$data; $i++) {
		my $firstletter = lc(substr($data->[$i]->{$flfield}, 0, 1));
		if ($firstletter =~ /[a-z]/) {
			push @{$byletter->{$firstletter}}, \$data->[$i];
		} else {
			push @{$byletter->{'#'}}, \$data->[$i];
		}
	}
	foreach ('a'..'z') {
		push @{$letters}, {
			display_letter=> $_,
			param_letter => $_,
			is_link => defined $byletter->{lc($_)}
			};
	}
	push @$letters, {
		display_letter => '0-9', 
		param_letter => '0', 
		is_link => defined $byletter->{'#'}
		};
	push @$letters, {
		display_letter => 'tout',
		param_letter => undef,
		is_link => 1
		};
	return {
		data => $data,
		byletter => $byletter,
		letters => $letters,
		sortby => $sortby,
		flfield => $flfield 
		};
}

####################################################################
####################################################################
## sub getpage                                                    ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $page : la page  afficher                                   ##
##   $letter : la lettre en cours ou undef                        ##
##   $perpage : le nombre de lignes dans le tableau               ##
##   $drawlettersthreshold :  partir de combien d'entres faut   ##
##                           il afficher les lettres              ##
##                                                                ##
## Retour :                                                       ##
##   un template  parser                                         ##
####################################################################
####################################################################

sub getpage($$$$$)
{
	my ($values, $page, $letter, $perpage, $drawlettersthreshold) = @_;
	my $source;

	$page = 1 if (!defined $page);
	if (defined $letter && (defined $values->{byletter}->{lc($letter)}))
	{
		$source = $values->{byletter}->{lc($letter)};
	}
	else
	{
		$source = $values->{data};
	}
	return if (!defined $source);
	my $num = scalar @$source;
	my $maxpage = int ($num / $perpage) + (($num % $perpage) != 0);
	if ($page > $maxpage)
	{
		$page = $maxpage; 
	}
	if ($page < 1)
	{
		$page = 1;
	}
	my $tags;
	$tags->{page} = $page;
	$tags->{maxpage} = $maxpage;
	$tags->{first} = 1 if ($page != 1);
	$tags->{last} = $maxpage if ($page != $maxpage);
	$tags->{letters} = $values->{letters};
	$tags->{display_letters} = ($num >= $drawlettersthreshold) || ((defined $letter) && ($letter ne ""));
	$tags->{prev} = $page - 1 if ($page > 1);
	$tags->{next} = $page + 1 if ($page < $maxpage);
	if ($num)
	{
		$tags->{lines} = [];
		for (my $i = 0; ((($page - 1) * $perpage + $i) < $num) && ($i < $perpage); $i++)
		{
			my $entry = $source->[($page - 1) * $perpage + $i];
			if (ref $entry eq "REF")
			{
				push @{$tags->{lines}}, Rewrite_Table($tablefile, $tablename, $$entry);
			}
			else
			{
				push @{$tags->{lines}}, Rewrite_Table($tablefile, $tablename, $entry);
			}
		}
		$tags->{headers} = @{$tags->{lines}}[0]->{valeurs};
		$tags->{title} = @{$tags->{lines}}[0]->{titre};
	}
	return remplace($tpl, $tags);
}

####################################################################
####################################################################
## sub addentry                                                   ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $entry : l'entree  ajouter                                  ##
##                                                                ##
## Retour :                                                       ##
##   le nouveau HREF pour l annuaire                              ##
####################################################################
####################################################################

sub addentry($$)
{
  my ($values, $entry) = @_;

  push @{$values->{data}}, $entry;
  return (initdata($values->{data}, $values->{sortby}, $values->{flfield}));
}

####################################################################
####################################################################
## sub delentry                                                   ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $field : le champs sur lequel faire la recherche             ##
##   $val : la valeur du champs                                   ##
##                                                                ##
## Retour :                                                       ##
##   le nouveau HREF pour l annuaire                              ##
####################################################################
####################################################################

sub delentry($$$)
{
  my ($values, $field, $val) = @_;

  for (my $i = 0; $i < scalar @{$values->{data}}; $i++)
  {
    if (@{$values->{data}}[$i]->{$field} eq $val)
    {
      for (my $j = $i + 1; $j < scalar @{$values->{data}}; $j++)
      {
        @{$values->{data}}[$j - 1] = @{$values->{data}}[$j];
      }
      pop @{$values->{data}};
      return (initdata($values->{data}, $values->{sortby}, $values->{flfield}));
    }
  }
  return ($values);
}

####################################################################
####################################################################
## sub modifyentry                                                ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $field : le champs sur lequel faire la recherche             ##
##   $val : la valeur du champs                                   ##
##   $entry : la nouvelle entre                                  ##
##                                                                ##
## Retour :                                                       ##
##   le nouveau HREF pour l annuaire                              ##
####################################################################
####################################################################

sub modifyentry($$$$)
{
  my ($values, $field, $val, $entry) = @_;

  for (my $i = 0; $i < scalar @{$values->{data}}; $i++)
  {
    if (@{$values->{data}}[$i]->{$field} eq $val)
    {
      @{$values->{data}}[$i] = $entry;
      return (initdata($values->{data}, $values->{sortby}, $values->{flfield}));
    }
  }
  return ($values);
}

####################################################################
####################################################################
## sub getentry                                                   ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $field : le champs sur lequel faire la recherche             ##
##   $val : la valeur du champs                                   ##
##                                                                ##
## Retour :                                                       ##
##   le HREF trouv ou undef                                      ##
####################################################################
####################################################################

sub getentry($$$)
{
  my ($values, $field, $val) = @_;
  foreach (@{$values->{data}})
  {
    if ($_->{$field} eq $val)
    {
      return ($_);
    }
  }
  return ;
}

####################################################################
####################################################################
## sub searchentry                                                ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##   $field : le champs sur lequel faire la recherche             ##
##   $val : la valeur du champs (une regexp)                      ##
##                                                                ##
## Retour :                                                       ##
##   le tableau contenant les HREF trouvs ou undef               ##
####################################################################
####################################################################

sub searchentry($$$)
{
   my ($values, $field, $val) = @_;

   my $res = undef;

   my ($re, $flags) = $val =~ /^\/(.*)\/([^\/]*)$/o;
   $re = $flags ? eval("qr/$re/$flags") : qr/$re/;
   for (my $i = 0; $i < scalar @{$values->{data}}; $i++)
   {
     if (@{$values->{data}}[$i]->{$field} =~ /$re/)
     {
       push @$res, @{$values->{data}}[$i];
     }
   }
   return ($res);
}

####################################################################
####################################################################
## sub countentries                                               ##
##                                                                ##
## Parametres :                                                   ##
##   $values : le HREF renvoy par initdata                       ##
##                                                                ##
## Retour :                                                       ##
##   le nombre d'entrees dans le cache                            ##
####################################################################
####################################################################

sub countentries($)
{
  my ($values) = @_;

  if ((!defined $values) || (!defined $values->{data}))
  {
    return ;
  }
  return (scalar @{$values->{data}});
}

1;
